import os
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QSize
from PySide2.QtGui import QIcon, QImage
from PySide2.QtSvg import QSvgGenerator
from PySide2.QtWidgets import QDialog, QWidget, QFileDialog
from axipy import LegendView, CurrentSettings

from ExportLegend.ui.utils.ui_tools import load_ui


class ExportLegend(QDialog):
    def __init__(self,parent=None):
        super().__init__(parent)
        path_ui = os.path.join(os.path.dirname(__file__), "DlgExportLegend.ui")

        self.ui=load_ui(path_ui, parent)
        self.ui.setWindowFlags(
            self.ui.windowFlags()  & ~QtCore.Qt.WindowContextHelpButtonHint)
        self.ui.pb_run.clicked.connect(self.__run_export)
        self.ui.pb_close.clicked.connect(self.__close)
        self.ui.finished.connect(self.__close)
        self.ui.tbsaveas.clicked.connect(self.__saveAs)
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")

        path_save_png = os.path.join(path_icon, "saveas.png")
        self.ui.tbsaveas.setIcon(QIcon(path_save_png))
        self.__init_dpi()
        self.ui.dpi.valueChanged.connect(self.__change_dpi)
    def __init_dpi(self):
        main_win = axipy.mainwindow
        qt_main = main_win.qt_object()
        self.__dpi=int(qt_main.screen().physicalDotsPerInch())
        self.ui.dpi.setValue(self.__dpi)
    def __prepare(self,legenView:LegendView):
        self.__legend=legenView
        width=legenView.widget.size().width()
        height=legenView.widget.size().height()
        self.ui.sp_width.setValue(width)
        self.ui.sp_height.setValue(height)
        last_path=CurrentSettings.LastOpenPath
        title=legenView.title
        title=title.replace(':','_').replace(' ','_')
        path_out=last_path.joinpath(title+".png")
        self.ui.path_image.setText(str(path_out))

    def __change_dpi(self,value_dpi):
        coef=value_dpi/self.__dpi
        new_width=int(coef*self.ui.sp_width.value()+0.5)
        new_height = int(coef * self.ui.sp_height.value() + 0.5)
        self.ui.sp_width.setValue(new_width)
        self.ui.sp_height.setValue(new_height)

    def __saveAs(self):
        fileName, ext_file = QFileDialog.getSaveFileName(self.ui,"Файл растра", self.ui.path_image.text(), "Png(*.png);;Tiff(*.tif);;Scalable Vector Graphics (*.svg)")
        #fileName, ext_file = QFileDialog.getSaveFileName(self.ui, "Файл растра", self.ui.path_image.text(),
        #                                                 "Png(*.png);;Tiff(*.tif)")
        if fileName!='':
            self.ui.path_image.setText(fileName)
    def __run_export(self):
        w=self.ui.sp_width.value()
        h=self.ui.sp_height.value()
        w_old = self.__legend.widget.width()
        h_old = self.__legend.widget.height()
        self.__legend.widget.resize(w, h)
        format_img = "PNG"

        ext = str(Path(self.ui.path_image.text()).suffix)
        ext = ext.replace('.', '').upper()
        if ext != format_img:
            format_img = ext
        if format_img=="SVG":
            generatorSvg = QSvgGenerator()
            generatorSvg.setSize(self.__legend.widget.size())
            #path_file = r"E:\Support\AxiomaTesting\PanoramaRsc\testTerplan6\leg_16.svg"
            path_file=self.ui.path_image.text()
            #path_file = r"E:\Support\AxiomaTesting\PanoramaRsc\testTerplan6\leg_7.svg"
            generatorSvg.setFileName(path_file)
            self.__legend.widget.render(generatorSvg)
            self.__legend.widget.resize(w_old, h_old)
            self.ui.close()
            return

        image_leg = QImage(w,h, QImage.Format.Format_ARGB32)
        image_leg.setDotsPerMeterX(int(39.37007874*self.ui.dpi.value()))
        image_leg.setDotsPerMeterY(int(39.37007874 * self.ui.dpi.value()))

        self.__legend.widget.render(image_leg)
        self.__legend.widget.resize(w_old, h_old)

        res=image_leg.save(self.ui.path_image.text(),format_img)
        self.ui.close()



    def __close(self):
        self.ui.close()
    def show(self,legenView:LegendView):
        self.__prepare(legenView)
        self.ui.show()

